﻿using System;
using System.Drawing;
using System.Collections;
using System.Collections.ObjectModel;
using System.IO;
using ThinkGeo.MapSuite.Core;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using GeoBrowser.ImageryServiceReference;
using System.Diagnostics;
//using Framework.Common.Classes.Logging;
//using Framework.Common.Classes.Exceptions;

namespace GeoBrowser
{
    public class ImageRasterLayer : RasterLayer
    {
        private string _layerType;

        public ImageRasterLayer(string layerType)
        {
            _layerType = layerType;
        }
        //
        // Summary:
        //     This property returns true if the RasterLayer is open and false if it is
        //     not.
        //
        // Remarks:
        //     Various methods on the RasterLayer require that it be in an open state. If
        //     one of those methods is called when the state is not open, then the method
        //     will throw an exception. To enter the open state, you must call the RasterLayer's
        //     Open method. The method will raise an exception if the current RasterLayer
        //     is already open.
        protected override bool IsOpenCore
        {
            get { return true; }
        }
        // Summary:
        //     This method opens the RasterLayer so that it is initialized and ready to
        //     use.
        //
        // Returns:
        //     None
        //
        // Remarks:
        //     The close method plays an important role in the life cycle of the RasterLayer.
        //     It may be called after drawing to release any memory and other resources
        //     that were allocated since the Open method was called.  If you override this
        //     method, it is recommended that you take the following things into account:
        //     This method may be called multiple times, so we suggest you write the method
        //     so that that a call to a closed RasterLayer is ignored and does not generate
        //     an error.  We also suggest that in the Close you free all resources that
        //     have been opened.  Remember that the object will not be destroyed, but will
        //     be re-opened possibly in the near future.
        protected override void CloseCore()
        {
        }
        //
        // Summary:
        //     This method will draw the image from the GeoImage source based on the parameters
        //     provided.
        //
        // Parameters:
        //   canvas:
        //     This parameter is the GeoCanvas used to Draw the RasterLayer.
        //
        //   labelsInAllLayers:
        //     This parameter is not used for ImageLayers.
        //
        // Returns:
        //     None
        //
        // Remarks:
        //     The DrawCore method will be called when the layer is being drawn. It will
        //     determine if the image is within the extent and the threshold defined. If
        //     these parameters are met, it will query the RasterSource for an image and
        //     then apply the other various properties (such as transparency, etc.) on the
        //     image. Lastly, it will draw the image on the GeoImage or native image passed
        //     into the method.
        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            //Logger.WriteLine("In DrawCore");

            Stopwatch stopwatch = new Stopwatch();

            stopwatch.Start();
            ExtentInfo extentInfo = new ExtentInfo();
            extentInfo.UpperLeft = new Coord();
            extentInfo.UpperLeft.X = canvas.CurrentWorldExtent.UpperLeftPoint.X;
            extentInfo.UpperLeft.Y = canvas.CurrentWorldExtent.UpperLeftPoint.Y;
            extentInfo.LowerRight = new Coord();
            extentInfo.LowerRight.X = canvas.CurrentWorldExtent.LowerRightPoint.X;
            extentInfo.LowerRight.Y = canvas.CurrentWorldExtent.LowerRightPoint.Y;
            extentInfo.Height = (int)canvas.Height;
            extentInfo.Width = (int)canvas.Width;

            ImageryServiceClient imageryServiceClient = null;
            ImageInfo imageInfo = null;
            try
            {
                imageryServiceClient = new ImageryServiceClient();

                ImageTheme imageTheme = ImageTheme.Satellite;

                if (_layerType == "Hybrid")
                {
                    imageTheme = ImageTheme.Hybrid;
                }
                imageInfo = imageryServiceClient.GetImage(extentInfo, imageTheme, ServiceLevel.Standard);
            }
            catch (Exception ex)
            {
                //ExceptionHandler.LogException(ex);
            }
            finally
            {
                if (imageryServiceClient != null && imageryServiceClient.State == System.ServiceModel.CommunicationState.Opened)
                {
                    imageryServiceClient.Close();
                }
            }
            stopwatch.Stop();

            if (imageInfo != null)
            {
                //Logger.WriteLine("Got {0} bytes of image in {1} milliseconds", imageInfo.Image.Length,
                //    stopwatch.ElapsedMilliseconds);

                Image jpegImage = byteArrayToImage(imageInfo.Image);

                //jpegImage.Save("C:\\bits\\" + Guid.NewGuid().ToString() + ".jpeg");
                MemoryStream memoryStream = new MemoryStream();
                jpegImage.Save(memoryStream, System.Drawing.Imaging.ImageFormat.Tiff);

                jpegImage.Save("C:\\bits\\" + Guid.NewGuid().ToString() + ".tiff");

                GeoImage customImage = new GeoImage(memoryStream);

                canvas.DrawWorldImageWithoutScaling(
                    customImage,
                    canvas.CurrentWorldExtent.GetCenterPoint().X,
                    canvas.CurrentWorldExtent.GetCenterPoint().Y,
                    DrawingLevel.LevelOne);

                customImage.Dispose();

#if false
                canvas.DrawScreenImageWithoutScaling(geoImage,
                    0,
                    0,
                    DrawingLevel.LevelOne, 0, 0, 0);
#endif
            }
        }

        public static Image byteArrayToImage(byte[] byteArrayIn)
        {
            MemoryStream ms = new MemoryStream(byteArrayIn);
            Image returnImage = Image.FromStream(ms);
            return returnImage;
        }
        //
        // Summary:
        //     This method returns the bounding box of the RasterLayer.
        //
        // Returns:
        //     This method returns the bounding box of the RasterLayer.
        //
        // Exceptions:
        //   System.InvalidOperationException:
        //     In the event you attempt to call this method on a image layer which has not
        //     been opened, it will throw an InvalidOperationException.
        //
        // Remarks:
        //     This method returns the bounding box of the RasterLayer.
        protected override RectangleShape GetBoundingBoxCore()
        {
            return new RectangleShape();
        }

        //
        // Summary:
        //     This method opens the RasterLayer so that it is initialized and ready to
        //     use.
        //
        // Returns:
        //     None
        //
        // Exceptions:
        //   System.InvalidOperationException:
        //     In the event you attempt to call this method on a image source which has
        //     already been opened, it will throw an InvalidOperationException.
        //
        // Remarks:
        //     The open method plays an important rol,e as it is responsible for initializing
        //     the RasterLayer. Most methods on the RasterLayer will throw an exception
        //     if the state of the RasterLayer is not opened. When the map draws each layer,
        //     it will open the RasterLayer as one of its first steps; then, after it is
        //     finished drawing with that layer, it will close it. In this way, we are sure
        //     to release all resources used by the RasterLayer.
        protected override void OpenCore()
        {
        }
    }
}
